require("defines")

IRON_STICK_STACK_SIZE = 100

script.on_init(function()
	for k,force in pairs(game.forces) do
		if force.technologies["basic-exoskeleton-equipment"].researched == true then
			force.technologies["cursed-jp"].enabled = true
		end
	end
	global.ironStickCount = {}
end)

script.on_event(defines.events.on_tick, function(event)
	for inv = defines.inventory.player_main, defines.inventory.player_trash do
		if global.ironStickCount ~= nil and global.ironStickCount[inv] ~= nil and global.ironStickCount[inv] > 0 then
			game.player.character.get_inventory(inv).remove{name="iron-stick", count=global.ironStickCount[inv]}
			global.ironStickCount[inv] = 0
		end
	end
	if event.tick % 60 == 0 then
		for _,player in ipairs(game.players) do
			local jetpackAvailable = false
			if not (player.character.cursor_stack.valid_for_read and player.character.cursor_stack.type == "armor") then
				local armor = player.get_inventory(defines.inventory.player_armor)[1]
				if armor.valid_for_read and armor.has_grid then
					for k,v in pairs(armor.grid.equipment) do
						if v.name == "cursed-jp" and v.energy == v.max_energy then
							jetpackAvailable = true
						end
					end
				end
			end
			if string.sub(player.character.name,-3) ~= "_jp" then
				if jetpackAvailable then
					changeBody(player)
				end
			else
				if not jetpackAvailable then
					changeBody(player)
				end
			end
		end
	end
end)

function changeBody(player)
	local old = player.character
	local new
	if string.sub(player.character.name,-3) ~= "_jp" then
		new = player.surface.create_entity{name=player.character.name .. "_jp", position=player.position, force=player.force}
	else
		new = player.surface.create_entity{name=string.sub(player.character.name,1,-4), position=player.position, force=player.force}
	end
	for inv = defines.inventory.player_main, defines.inventory.player_trash do
		if inv ~= defines.inventory.player_vehicle then
			moveInside(old,new,inv)
		end
	end

	local slots = player.force.character_logistic_slot_count
	if slots > 0 then
		for c=1, slots do
			local req = old.get_request_slot(c)
			if req ~= nil then
				new.set_request_slot(req,c)
			end
		end
	end

	local cursor = old.cursor_stack
	player.set_controller({type = defines.controllers.character, character = new})
	player.character.cursor_stack.set_stack(cursor)

	old.destroy()
end

function moveInside(oldChar,newChar,loc)
	local oldInv = oldChar.get_inventory(loc)
	local newInv = newChar.get_inventory(loc)

	if global.ironStickCount == nil then
		global.ironStickCount = {}
	end
	global.ironStickCount[loc] = 0

	for i = 1, #oldInv do
		if oldInv[i].valid_for_read then
			newInv.insert(oldInv[i])
			if game.item_prototypes[oldInv[i].name].type == "armor" and oldInv[i].has_grid and #oldInv[i].grid.equipment ~= 0 then
				for j = 1, #newInv do
					if newInv[j].valid_for_read and newInv[j].name == oldInv[i].name and newInv[j].has_grid and #newInv[j].grid.equipment == 0 then
						local inside = oldInv[i].grid.equipment
						for k = 1, #inside do
							newInv[j].grid.put(inside[k])
						end
						return
					end
				end
			end
		else
			newInv.insert{name="iron-stick", count=IRON_STICK_STACK_SIZE}
			global.ironStickCount[loc] = global.ironStickCount[loc]+IRON_STICK_STACK_SIZE
		end
	end
end